@@ -1114,7 +1114,7 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
               if (count == 0)
                 break;
               buffer[count]='\0';
-              if ((count+offset+MagickPathExtent) >= (ssize_t) extent)
+              if (((ssize_t) count+offset+MagickPathExtent) >= (ssize_t) extent)
                 {
                   extent<<=1;
                   comments=(char *) ResizeQuantumMemory(comments,extent+
@@ -1289,7 +1289,10 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       ((size_t) (flag & 0x07)+1);
     image->colors=local_colors;
     if (opacity >= (ssize_t) image->colors)
-      opacity=(-1);
+      {
+        image->colors++;
+        opacity=(-1);
+      }
     image->ticks_per_second=100;
     image->matte=opacity >= 0 ? MagickTrue : MagickFalse;
     if ((image->columns == 0) || (image->rows == 0))
